# usage:
#    props2csv.py -d <directory> 
#    
#    <directory> is a folder containing .fzp files.  
#    save a csv file of props, tags, etc. to <csv file path> 

import getopt, sys, os, re, csv, xml.dom.minidom, xml.dom
    
def usage():
    print """
usage:
    props2csv.py -d <directory> -c <csv file path>
    
    <directory> is a folder containing .fzp files.  
    save a csv file of props, tags, etc. to <csv file path> 
    """
        
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:c:", ["help", "directory", "csv"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputDir = None
    csvPath = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-c", "--csv"):
            csvPath = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if not outputDir:
        usage()
        sys.exit(2)
        
    if not csvPath:
        usage()
        sys.exit(2)
        
    writer = None
    file = None
    try:
        file = open(csvPath, 'wb')
        writer = csv.writer(file, delimiter=',')
        writer.writerow(["fzp","location","title","description","family","props","tags","taxonomy"] )
    except:
        print "unable to save to", csvPath
        sys.exit(2)
            
    names = []
    for filename in os.listdir(outputDir): 
        if (filename.endswith(".fzp")):  
            fzpFilename = os.path.join(outputDir, filename)
            try:
                dom = xml.dom.minidom.parse(fzpFilename)
            except xml.parsers.expat.ExpatError, err:
                print str(err), fzpFilename
                continue
             
            theLine = filename + ","
            
            fzp = dom.documentElement
            
            titleText = ""
            titles = fzp.getElementsByTagName("title")
            for title in titles:
                titleText = getText(title.childNodes)
                break       # assume only one title
                
            taxonomyText = ""
            taxonomies = fzp.getElementsByTagName("taxonomy")
            for taxonomy in taxonomies:
                taxonomyText = getText(taxonomy.childNodes)
                break       # assume only one title
                
            location = "core"
            if "contrib" in fzpFilename:
                location = "contrib"
            elif "resource" in fzpFilename:
                location = "resources"
            
            descriptionText = ""
            descriptions = fzp.getElementsByTagName("description")
            for description in descriptions:
                descriptionText = getText(description.childNodes)
                break       # assume only one description
            
            tagsText = ""
            tags = fzp.getElementsByTagName("tag")
            for tag in tags:
                tagsText += getText(tag.childNodes) + "\n"

            familyText = ""
            propertiesText = ""
            properties = fzp.getElementsByTagName("property")
            for property in properties:
                name = property.getAttribute('name')
                value = getText(property.childNodes)
                propertiesText += name + ":" + value + "\n"
                if name == "family":
                    familyText = value
            
            writer.writerow([filename, location, titleText.encode("utf-8"), descriptionText.encode("utf-8"), familyText.encode("utf-8"), propertiesText.encode("utf-8"), tagsText.encode("utf-8"), taxonomyText.encode("utf-8")])
    
    if file:
        file.close()
        
            
def getText(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc) 
    
if __name__ == "__main__":
    main()




# usage:
#    grep.py -d <directory> -f <text>
#    
#    <directory> is a folder, with subfolders, containing .svg files.  In each svg file in the directory or its children
#    look for <text>

import getopt, sys, os, re
    
def usage():
    print """
usage:
    grep.py -d [directory] -f [text] -n
    
    directory is a folder containing .svg files.  
    In each svg file in the directory or its subfolders,
    look for [text]
    """
    
  	
	   
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hnd:f:", ["help", "not", "directory", "find"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputDir = None
    findtext = ""
    gotnot = 0
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        elif o in ("-f", "--find"):
            findtext = a;
        elif o in ("-n", "--not"):
            gotnot = 1;

        else:
            assert False, "unhandled option"
    
    if(not(outputDir)):
        usage()
        sys.exit(2)
     

    print "finding text " + findtext   
    
    for root, dirs, files in os.walk(outputDir, topdown=False):
        for filename in files:
            if (filename.endswith(".svg")):  
                infile = open(os.path.join(root, filename), "r")
                svg = infile.read();
                infile.close();

                rslt = svg.find(findtext)

                if gotnot and (rslt < 0):
                    print "{0}:{1}".format(os.path.join(root, filename), rslt)
                if (not gotnot) and (rslt > -1):
                    print "{0}:{1}".format(os.path.join(root, filename), rslt)

               
  
    
if __name__ == "__main__":
    main()




# usage:
#	thtsms.py -d [fzp folder] -t [tht | smd]
#       add or update a package <property> element.

import getopt, sys, os, os.path, re, xml.dom.minidom, xml.dom
    
def usage():
        print """
usage:
    thtsmd.py -d [fzp folder] -t [tht | smd]
    add or update a package <property> element.
"""
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:t:", ["help", "directory", "type"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
        
    dir = None
    typ = None
            
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            dir = a
        elif o in ("-t", "--type"):
            typ = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
            
    if not(dir):
        usage()
        sys.exit(2)    
        
    if not(typ):
        usage()
        sys.exit(2)       
    
    for root, dirs, files in os.walk(dir, topdown=False):
        for filename in files:
            if not filename.endswith(".fzp"): 
                continue
                
            fzpFilename = os.path.join(root, filename)
            infile = open(os.path.join(root, filename), "r")
            content = infile.read()
            infile.close()
            try:
                dom = xml.dom.minidom.parseString(content)
            except xml.parsers.expat.ExpatError, err:
                print str(err), fzpFilename
                continue
                
            #print "testing", fzpFilename
                
            fzp = dom.documentElement
            properties = None
            propertiesNodes = fzp.getElementsByTagName("properties")
            if (propertiesNodes.length == 0):
                properties = dom.createElement("properties")
                fzp.insertBefore(properties, fzp.firstChild)
            elif (propertiesNodes.length == 1):
                properties = propertiesNodes.item(0)                
            else:
                print "multiple properties elements in", fzpFilename
                continue
             

            #print "got properties", properties
            
            propertyNodes = properties.getElementsByTagName("property")
            package = None
            for property in propertyNodes:
                name = property.getAttribute("name")
                #print "got name", name
                if name == "package":
                    package = property
                    break
 
            #print "got package", package
 
            if package == None:
                package = dom.createElement("property")
                properties.appendChild(package)
                package.setAttribute("name", "package")
                t = dom.createTextNode(typ)
                package.appendChild(t)
            else:
                package.normalize()  #make sure all the text is in one node
                t = None
                for child in package.childNodes:
                    if child.nodeType == child.TEXT_NODE:
                        t = child
                        break                        
                
                if t == None:
                    t = dom.createTextNode(typ)
                    package.appendChild(t)
                else:
                    if ("SMD" in t.nodeValue):
                        continue
                    if ("THT" in t.nodeValue):
                        continue
                    
                    t1 = dom.createTextNode(" [" + typ + "]")
                    package.appendChild(t1)
                    package.normalize()
            
            print "adding package property " + typ + " to", fzpFilename
            outfile = open(fzpFilename, 'wb')
            s = dom.toxml("UTF-8")
            outfile.write(s)
            outfile.flush()
            outfile.close()                    
            
if __name__ == "__main__":
        main()






#!/usr/bin/python
# -*- coding: utf-8 -*-

# lots of borrowing from http://code.activestate.com/recipes/252508-file-unzip/

import getopt, sys, os, os.path, re, zipfile, shutil, xml.dom.minidom, xml.dom
import xml.etree.ElementTree as ET
    
def usage():
    print """
usage:
    checkino.py -f [from directory] -e [=yes extract .ino files]

For each fzz file in the from directory, see whether the fz specifies an ino file and if it matches the ino file name.  Probably safest to make a copy of the from directory first
"""
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "e:f:", ["extract", "pull"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    
    inputDir = None
    extract = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--from"):
            inputDir = a
        elif o in ("-e", "--extract"):
            extract = a
        else:
            print "unhandled option",o
            usage()
            return
    
    if not(inputDir):
        print "missing 'from' argument"
        usage()
        return
        
    try:
        import zlib
        compression = zipfile.ZIP_DEFLATED
    except:
        compression = zipfile.ZIP_STORED

    for root, dirs, files in os.walk(inputDir, topdown=False):
        for fzz in files:
            if not fzz.endswith('.fzz'):
                continue
                
            #print fzz
            fzzpath = os.path.join(root, fzz)
            
            tempDir = inputDir + os.sep + "___temp___"
            shutil.rmtree(tempDir, 1)
            os.mkdir(tempDir)

            zf = zipfile.ZipFile(fzzpath)
            zf.extractall(tempDir)
            zf.close()
            
            fzzbase = os.path.splitext(fzz)[0]
            
            renamed = False
            
            inos = []
            for fz in os.listdir(tempDir):
                if fz.endswith(".ino"):
                    if extract == "yes":
                        inoPath = os.path.join(tempDir, fz)
                        shutil.copyfile(inoPath, os.path.join(inputDir, fz))
                        print "copy", inoPath, os.path.join(inputDir, fz)
                    inos.append(fz)
                    continue
                    
                if not fz.endswith(".fz"):
                    continue
                    
                try: 
                    fzpath = os.path.join(tempDir, fz)
                    tree = ET.parse(fzpath)
                    top = tree.getroot()
                    for program in top.iter('program'):
                        print "program", program.text, fzzpath
                        
                except:
                    print "exception", fzpath, sys.exc_info()[0]
                    return    

            for ino in inos:
                print "    ino", ino, fzzpath

            if renamed:
                os.remove(fzzpath)
                zf = zipfile.ZipFile(fzzpath, mode='w')
                for fn in os.listdir(tempDir):
                    zf.write(os.path.join(tempDir, fn), fn, compression)
                zf.close()

            shutil.rmtree(tempDir, 1)

if __name__ == "__main__":
    main()







import getopt, sys, os, os.path, re, xml.dom.minidom, xml.dom
    
def usage():
        print """
usage:
    unusedsvgs.py -f [fzp folder] -s [svg folder]
    lists orphan svgs
"""
    
        
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:s:", ["help", "fzp", "svg"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
        
    fzpdir = None
    svgdir = None
            
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--fzp"):
            fzpdir = a
        elif o in ("-s", "--svg"):
            svgdir = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            print "unhandled option", o
            return
            
    if not fzpdir:
        print "missing fzp folder argument"
        usage()
        return           
            
    if not svgdir:
        print "missing svg folder argument"
        usage()
        return  

    svgfiles = {}
    for root, dirs, files in os.walk(svgdir, topdown=False):
        for filename in files:
            if not filename.endswith(".svg"): 
                continue
            
            basename = os.path.basename(root)
            if svgfiles.get(basename) == None:
                svgfiles[basename] = []
                
            svgfiles[basename].append(filename)
            
    viewnames = ["iconView", "breadboardView", "schematicView", "pcbView"]
            
    for root, dirs, files in os.walk(fzpdir, topdown=False):
        for filename in files:
            if not filename.endswith(".fzp"): 
                continue
                
            fzpFilename = os.path.join(root, filename)
            try:
                dom = xml.dom.minidom.parse(fzpFilename)
            except xml.parsers.expat.ExpatError, err:
                print str(err), fzpFilename
                continue
                
            fzp = dom.documentElement
            for viewname in viewnames:
                viewNodes = fzp.getElementsByTagName(viewname)
                for viewNode in viewNodes:
                    layersNodes = viewNode.getElementsByTagName("layers")
                    for layersNode in layersNodes:
                        image = layersNode.getAttribute("image")
                        dn = os.path.dirname(image)
                        viewFiles = svgfiles.get(dn)
                        if viewFiles:
                            fn = os.path.basename(image)
                            try:
                                if fn in viewFiles:
                                    print "{0} uses {1}/{2}".format(os.path.basename(root), dn, fn)
                                    viewFiles.remove(fn)
                            except:
                                pass
                        
                        
    for key in svgfiles.keys():
        for name in svgfiles.get(key):
            print "unused {0}/{1}".format(key, name)

            
if __name__ == "__main__":
        main()






#!/usr/bin/env python
# encoding: utf-8
"""
/*******************************************************************

Part of the Fritzing project - http://fritzing.org
Copyright (c) 2007-08 Fachhochschule Potsdam - http://fh-potsdam.de

Fritzing is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fritzing is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************

$Revision: 1595 $:
$Author: dirk@fritzing.org $:
$Date: 2008-11-21 11:04:36 +0100 (Fri, 21 Nov 2008) $

********************************************************************/

valuesAndColors_forResistors.py

Specific Script to generate a set of Resistors

"""

import sys
import os

E3Set = [100, 220, 470]
E6Set = [100, 150, 220, 330, 470, 680]
E12Set = [100, 120, 150, 180, 220, 270, 330, 390, 470, 560, 680, 820]
E24Set = [100, 110, 120, 130, 150, 160, 180, 200, 220, 240, 270, 300, 330, 360, 390, 430, 470, 510, 560, 620, 680, 750, 820, 910]

NamedSets = {'e3': E3Set, 'e6': E6Set, 'e12': E12Set, 'e24': E24Set}
 
SetMultiplication = [0.1, 1, 10, 100, 1000, 10000]
letterDict = { 'k': 1000, 'M': 1000000, 'G': 1000000000}

colorBands = ['black', 'brown', 'red', 'orange', 'yellow', 'green', 'blue', 'purple', 'grey', 'white']
colors = { 'black': (0, 0, 0),
		'brown': (138, 61, 6),
		'red': (196, 8, 8),
		'orange': (255, 77, 0),
		'yellow': (255, 213, 0),
		'green': (0, 163, 61),
		'blue': (0, 96, 182),
		'purple': (130, 16, 210),
		'grey': (140, 140, 140),
		'white': (255, 255, 255) }

def resistorSetForSetName(setName):
	if (setName[-3:].lower() == 'Set'.lower()):
		setName = setName[:-3]
	if NamedSets.has_key(setName.lower()):
		currentSet = NamedSets[setName]
		resistorSet = []
		for m in SetMultiplication:
			# if last of the set Multiplication
			if (m == SetMultiplication[len(SetMultiplication) - 1]):
				# only do the first
				value = currentSet[0] * m
				resistorSet.append(value)
			else:
				# do all in E set
				for r in currentSet:
					value = int(r * m)
					resistorSet.append(value)
		return resistorSet
	else:
		print >> sys.stderr, "No set with this name: %s (in %s)" % (setName, __file__)

def valueStringFromValue(value):
	short = False
	for k in reversed(letterDict.keys()):
		if value / letterDict[k]:
			short = True
			remainder = str(value % letterDict[k])
			remainder = remainder.rstrip('0')
			if remainder:
				return "%d.%d%s" % ((value / letterDict[k]), int(remainder), k)
			else:
				return "%d%s" % ((value / letterDict[k]), k)
			break
	if not short:
		return str(value)

def valueStringWithoutDots(value):
	return "_".join(valueStringFromValue(value).split("."))

def hexColorsForResistorValue(value):
	thirdBand = 0
	for m in range(6, 1, -1):
		if (value / pow(10, m)):
			thirdBand = m - 1
			break
	firstBand = colorBands[int(str(value)[:1])]
	secondBand = colorBands[int(str(value)[1:2])]
	thirdBand = colorBands[thirdBand]
	return ["#%02X%02X%02X" % (colors[c]) for c in (firstBand, secondBand, thirdBand)]

def main():
	resistorSet = resistorSetForSetName('E6Set')
	print resistorSet
	
	for r in resistorSet:
		print valueStringFromValue(r)
		print hexColorsForResistorValue(r)
	
	print "number of resistors in set: %d" % (len(resistorSet))
	for r in resistorSet:
		# Printing the value
		short = False
		for k in reversed(letterDict.keys()):
			if r / letterDict[k]:
				short = True
				remainder = str(r % letterDict[k])
				remainder = remainder.rstrip('0')
				print "spoken value: %d%s%s" % (r / letterDict[k], k, remainder)
				if remainder:
					print "official value: %d.%d%s" % ((r / letterDict[k]), int(remainder), k)
				else:
					print "official value: %d%s" % ((r / letterDict[k]), k)
				break
		if not short:
			print r
		# Printing the color bands
		thirdBand = 0
		for m in range(6, 1, -1):
			if (r / pow(10, m)):
				thirdBand = m - 1
				break
		firstBand = colorBands[int(str(r)[:1])]
		secondBand = colorBands[int(str(r)[1:2])]
		thirdBand = colorBands[thirdBand]
		print "%s, %s, %s" % (firstBand, secondBand, thirdBand)
		print "#%02X%02X%02X" % (colors[firstBand])
		print "#%02X%02X%02X" % (colors[secondBand])
		print "#%02X%02X%02X" % (colors[thirdBand])

	# for c in range(10):
	# 	print "#%02X%02X%02X" % (colors[colorBands[c]])


if __name__ == '__main__':
	main()





import getopt, sys, os, re, time
    
def usage():
    print """
usage:
    checkcopies.py -d <directory> 
    
    <directory> is a folder containing svg files.  
    looks for files that have the same content but different names
    
    """
    
    
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    
    outputDir = None
    gotnot = 0
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            assert False, "unhandled option"
    
    if(not(outputDir)):
        print "missing -d {directory} parameter"
        usage()
        return
     
    fs = []
    for f in os.listdir(outputDir):
        if f.endswith(".svg"):
            fs.append(os.path.splitext(f)[0])
            #print "appending", f, os.path.splitext(f)[0]
    fs.sort(key=str.lower)                                               # remove the .svg so the sort works better
    dirs = []
    for f in fs:
        dirs.append(f + ".svg")
        
    dirlen = len(dirs)
    matches = []
    for i in range(dirlen):
        if i in matches:
            continue
            
        f1 = dirs[i]
        path = os.path.join(outputDir, f1)
        if not os.path.isfile(path):
            continue
            
        if not f1.endswith(".svg"): 
            continue
            
        # print "testing", f1
            
        txt1 = None
        try:
            infile = open(path, "r")
            txt1 = infile.read()
            infile.close()
        except:
            print "failure", f1
            
        if txt1 == None:
            continue
             
        for j in range(i + 1, dirlen):
            f2 = dirs[j]
            path = os.path.join(outputDir,f2)
            if not os.path.isfile(path):
                continue
                
            if not f2.endswith(".svg"): 
                continue
                
            if f1 == f2:
                continue
                
            txt2 = None
            try:
                infile = open(path, "r")
                txt2 = infile.read()
                infile.close()
            except:
                print "failure", f2
                continue
                
            if txt2 == None:
                continue
                
            if txt1 == txt2:
                matches.append(j)
                print "<map package='{0}' to='{1}' />".format(f1, f2) 
    
if __name__ == "__main__":
    main()




# usage:
#    footprintgrid.py -o [outputfile] -u ['in' or 'mm' units] -l [left padding (units)] -t [top padding (units)] -r [right padding  (units)] -b [bottom padding  (units)] -x [horizontal pin count] -y [vertical pin count] -g [grid size (units)] -h [hole diameter (units)] -r [ring thickness (units)]
#   
#    writes an svg footprint file to [outputfile] 
#    units are either all in inches or millimeters
#    padding left and top is the distance from the edge of the part to the center of the top left pin
#    padding right and bottom is the distance from the edge of the part to the center of the bottom right pin
#    x and y define the number of pins
#    gridsize is the distance from the center of one pin to the center of another pin
#    drillhole is the diameter of the pin's drillhole
#    ring thickness is the width of the ring (not including the drillhole)
#
#    example: footprintgrid.py -o arduino_mini.svg -u in -l 0.1 -t 0.1 -r0.1 -b 0.1 -x 12 -y 7 -g 0.1 -c 0.02 -d 0.035
 



import getopt, sys, os, re
    
def usage():
    print """
usage:
    footprintgrid.py -o [outputfile] -u ['in' or 'mm' units] -l [left padding (units)] -t [top padding (units)] -r [right padding  (units)] -b [bottom padding  (units)] -x [horizontal pin count] -y [vertical pin count] -g [grid size (units)] -d [drillhole diameter (units)] -c [copper thickness (units)]
    
    writes an svg footprint file to [outputfile] 
    units are either all in inches or millimeters
    padding left and top is the distance from the edge of the part to the center of the top left pin
    padding right and bottom is the distance from the edge of the part to the center of the bottom right pin
    x and y define the number of pins
    g is the distance from the center of one pin to the center of another pin
    drillhole is the diameter of the pin's drillhole
    copper thickness is the width of the copper ring (not including the drillhole)
    """
    
  	
	   
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "ho:u:l:t:r:b:x:y:g:d:c:", ["help", "output", "units", "left", "top", "right", "bottom", "x", "y", "grid", "drillhole", "copper"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputFile = None
    units = None
    left = None
    right = None
    top = None
    bottom = None
    x = None
    y = None
    gridSize = None
    drillHole = None
    ringThickness = None
    
    for o, a in opts:
        if o in ("-o", "--output", "--outputfile"):
            outputFile = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        elif o in ("-u", "--units"):
            units = a
        elif o in ("-l", "--left"):
            left = float(a)
        elif o in ("-r", "--right"):
            right = float(a)
        elif o in ("-t", "--top"):
            top = float(a)
        elif o in ("-b", "--bottom"):
            bottom = float(a)
        elif o in ("-x"):
            x = int(a)
        elif o in ("-y"):
            y = int(a)
        elif o in ("-g", "--grid", "--gridsize"):
            gridSize = float(a)
        elif o in ("-d", "--drill", "--drillhole"):
            drillHole = float(a)
        elif o in ("-c", "--ring", "--ringthickness", "--thickness", "--copper", "--copperthickness"):
            ringThickness = float(a)
        else:
            print ("params %s %s" % (o, a))
            assert False, "unhandled option " 
    
    for thing in (outputFile, units, left, right, top, bottom, x, y, gridSize, drillHole, ringThickness):
        if(not(thing)):
            usage()
            sys.exit(2)
            
    if units == 'mm':
        left /= 25.4
        right /= 25.4
        top /= 25.4
        bottom /= 25.4
        gridSize /= 25.4
        drillHole /= 25.4
        ringThickness /= 25.4
    
    width = left + right + ((x - 1) * gridSize)
    width000 = width * 1000
    height = top + bottom + ((y - 1) * gridSize)
    height000 = height * 1000
    
    radius = (drillHole + ringThickness) / 2.0
        
    svg = "<?xml version='1.0' encoding='UTF-8' standalone='no'?>\n"
    svg += "<svg xmlns='http://www.w3.org/2000/svg' version='1.2' baseProfile='tiny' x='0in' y='0in' width='%fin' height='%fin' viewBox='0 0 %f %f'>\n" % (width, height, width000, height000)
    svg += "<g id='silkscreen'>\n"
    #across the top:
    svg += "<line x1='4' y1='4' x2='%f' y2='4' stroke='#ffffff' stroke-width='8' stroke-linecap='round' />\n" % (width000 - 8)
    #across the bottom:
    svg += "<line x1='4' y1='%f' x2='%f' y2='%f' stroke='#ffffff' stroke-width='8' stroke-linecap='round' />\n" % (height000 - 8, width000 - 8, height000 - 8)
    #down left with a break in the middle
    svg += "<line x1='4' y1='4' x2='4' y2='%f' stroke='#ffffff' stroke-width='8' stroke-linecap='round' />\n" % ((height000 / 2.0) - (1000 * gridSize / 2.0))
    svg += "<line x1='4' y1='%f' x2='4' y2='%f' stroke='#ffffff' stroke-width='8' stroke-linecap='round' />\n" % ((height000 / 2.0) + (1000 * gridSize / 2.0), height000 - 8)
    #down right
    svg += "<line x1='%f' y1='4' x2='%f' y2='%f' stroke='#ffffff' stroke-width='8' stroke-linecap='round' />\n" % (width000 - 8,  width000 - 8, height000 - 8)
    svg += "</g>\n"
    svg += "<g id='copper0'>\n"
    cy = top
    for iy in range(y):
        cx = left
        for ix in range(x):
            svg += "<circle id='connector%dpin' fill='none' cx='%f' cy='%f' r='%f' stroke='rgb(255, 191, 0)' stroke-width='%f' />\n" % (ix + ((y - iy -1) * x), cx * 1000, cy * 1000, radius * 1000, ringThickness * 1000)
            if (ix == 0 and iy == y - 1):
                svg += "<rect x='%f' y='%f' width='%f' height='%f' fill='none' stroke='rgb(255, 191, 0)' stroke-width='%f'/>\n" % ((cx - radius) * 1000, (cy - radius) * 1000, radius * 1000 * 2, radius * 1000 * 2, ringThickness * 1000)
            cx += gridSize
        
        cy += gridSize
        
    svg += "</g>\n"
    svg += "</svg>"
    infile = open(outputFile, "w")
    infile.write(svg);
    infile.close();
   
    
               
  
    
if __name__ == "__main__":
    main()




#!/usr/bin/env python
# encoding: utf-8
"""
/*******************************************************************

Part of the Fritzing project - http://fritzing.org
Copyright (c) 2007-08 Fachhochschule Potsdam - http://fh-potsdam.de

Fritzing is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Fritzing is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.

********************************************************************

$Revision: 1595 $:
$Author: dirk@fritzing.org $:
$Date: 2008-11-21 11:04:36 +0100 (Fri, 21 Nov 2008) $

********************************************************************/

resistorSetGenerator.py

"""

import sys, getopt, ConfigParser, os, uuid
from datetime import date
from Cheetah.Template import Template
from xml.dom import minidom


import valuesAndColors_forResistors

help_message = """
Usage:
	resistorSetGenerator.py -s [Resistor Set] -o [Output Dir] -b [Existing Resistor Bin]
	
	Resistor Set          - the named set of standard resistors that should be generated
					        Possible values are: E3, E6, E12, E24 for the E3Set, E6Set etc.
					
	Output Dir            - the location where the output files are written
	
	Existing Resistor Bin - the Fritzing bin file (.fzb) from which this script should try
	                        to find the matching moduleID's. If no bin is given, all new
	                        resistors with new moduleID's will be generated. 
"""


class Usage(Exception):
	def __init__(self, msg):
		self.msg = msg

def makeUUID():
	"creates an 8 character hex UUID"
	return str(uuid.uuid1())

def makeDate():
	"creates a date formatted as YYYY-MM-DD"
	return date.today().isoformat()

def getUserName():
	"gets the full user name if available"
	try:
		import pwd
		return pwd.getpwuid(os.getuid())[4]
	except:
		return os.getlogin()

def getTemplatefile(fileName):
	thisScriptPath = sys.argv[0]
	thisScriptFolder = os.path.split(thisScriptPath)[0]
	expectedTemplate = os.path.join(thisScriptFolder, fileName)
	if os.path.exists(expectedTemplate):
		return expectedTemplate
	else:
		print >> sys.stderr, "Script Package Incomplete: Couldn't find the svg template file: %s" % (fileName)
		sys.exit(2)

def getConfigFile(fileName):
	thisScriptPath = sys.argv[0]
	thisScriptFolder = os.path.split(thisScriptPath)[0]
	expectedTemplate = os.path.join(thisScriptFolder, fileName)
	if os.path.exists(expectedTemplate):
		return expectedTemplate
	else:
		print >> sys.stderr, "Script Package Incomplete: Couldn't load the config file: %s" % (fileName)
		sys.exit(2)

def getExistingResistorsFromBin(filename):
	if os.path.exists(filename):
		# Load the bin file as xml
		try:
			binFile = open(filename)
			xmldoc = minidom.parse(binFile).documentElement
			binFile.close()
		except (IOError, OSError):
			print >> sys.stderr, "Couldn't open bin: %s " % (fileName)
			sys.exit(2)
		# See if the bin got any resistors
		foundResistors = []
		for aPart in xmldoc.getElementsByTagName("instance"):
			# print aPart
			# print aPart.attributes.keys()
			if aPart.attributes.has_key("path") and aPart.attributes.has_key("moduleIdRef"):
				partFilename = os.path.split(aPart.attributes["path"].value)[1]
				if (partFilename.startswith("resistor_")):
					# There is a resistor. Strip of the resistor_ and .fzp part
					resistorValueString = os.path.splitext(partFilename)[0][9:]
					thisResistor = {}
					thisResistor['value'] = resistorValueString.replace("_", ".")
					thisResistor['moduleID'] = aPart.attributes["moduleIdRef"].value
					foundResistors.append(thisResistor)
			else:
				print "Couldn't determine following part in bin, because it has no path and moduleIdRef property: %s" % (aPart.toxml())
		return foundResistors
	else:
		print >> sys.stderr, "No Fritzing bin file: %s found" % (fileName)
		sys.exit(2)
	

def writeOutFileInSubDirectoryWithData(outputDir,  subDirectory, fileName, data):
	outputDir = os.path.join(outputDir, subDirectory)
	if not os.path.exists(outputDir):
		os.makedirs(outputDir)
	outfile = open(os.path.join(outputDir, fileName), "w")
	outfile.write(str(data))
	outfile.close()

def main(argv=None):
	if argv is None:
		argv = sys.argv
	try:
		try:
			opts, args = getopt.getopt(argv[1:], "ho:s:b:", ["help", "output=", "set=", "bin="])
		except getopt.error, msg:
			raise Usage(msg)
		
		output = "."
		verbose = False
		namedSet = None
		existingBin = None
	
		# option processing
		for option, value in opts:
			if option in ("-h", "--help"):
				raise Usage(help_message)
			if option in ("-o", "--output"):
				output = value
			if option in ("-s", "--set"):
				namedSet = value
			if option in ("-b", "--bin"):
				existingBin = value
		
		if(not(namedSet) or not(output)):
			# print >> sys.stderr, "No Resistor Set or Output Dir specified"
			raise Usage("Error: No Resistor Set or Output Dir specified")
		
		config = ConfigParser.ConfigParser()
		config.readfp(open(getConfigFile('defaults.cfg')))
		
		resistorSet = valuesAndColors_forResistors.resistorSetForSetName(namedSet)
		print "number of resistors in set: %d" % (len(resistorSet))
		
		# Analyzing bin for existing resistor moduleID's
		existingResistors = []
		if existingBin:
			print "analyzing bin for existing resistor moduleID's ..."
			existingResistors = getExistingResistorsFromBin(existingBin)
			existingResistorsValues = [r['value'] for r in existingResistors]
			
		resistorBin = []
		skipThisResistor = False
		for r in resistorSet:
			sL = {}
			metaData = {}
			resistorInBin = {}
			resistanceString = valuesAndColors_forResistors.valueStringFromValue(r)
			# check if this resistor doesn't have a moduleID already 
			if existingResistors and (resistanceString in existingResistorsValues):
				if (not config.getboolean('ExistingResistors', 'regenerateExitsingParts')):
					# Don't regnerate this resistor (leave it out the new set)
					print "Skipping %s resistor, because it is already in the existing bin" % (resistanceString)
					skipThisResistor = True
				i = existingResistorsValues.index(resistanceString)
				metaData['moduleID'] = existingResistors[i]['moduleID']
			else:
				metaData['moduleID'] = makeUUID()
			if (not skipThisResistor):
				metaData['date'] = makeDate()
				metaData['author'] = getUserName()
				sL['metaData'] = metaData
				sL['resistance'] = resistanceString
				# sL['taxonomy'] = "discreteParts.resistor.%s" % (resistanceString)
				# The resistance with the dot notation in the taxonomy might be a problem, so we'd better do:
				sL['taxonomy'] = "discreteParts.resistor.%d" % (r)
				sL['colorBands'] = valuesAndColors_forResistors.hexColorsForResistorValue(r)
				# Breadboard svg
				svg = Template(file=getTemplatefile("resistor_breadboard_svg.tmpl"), searchList = [sL])
				writeOutFileInSubDirectoryWithData(output, "breadboard", "resistor_%s.svg" % (resistanceString), svg)
				# Icon svg
				svg = Template(file=getTemplatefile("resistor_icon_svg.tmpl"), searchList = [sL])
				writeOutFileInSubDirectoryWithData(output, "icon", "resistor_icon_%s.svg" % (resistanceString), svg)
				# Partfile fzp
				fzpfilename = "resistor_%s.fzp" % valuesAndColors_forResistors.valueStringWithoutDots(r)
				fzp = Template(file=getTemplatefile("resistor_fzp.tmpl"), searchList = [sL])
				writeOutFileInSubDirectoryWithData(output, "fzp_files", fzfilename, fzp)
				# For in the bin
				resistorInBin['moduleID'] = metaData['moduleID']
				resistorInBin['filename'] = fzpfilename
				resistorBin.append(resistorInBin)
		# Bin file
		if (config.getboolean('OutputBin', 'outputBin')):
			sL = {}
			sL['title'] = "%s Resistor Set" % (namedSet)
			partsDir = config.get('OutputBin', 'partsDir')
			for r in resistorBin:
				r['filepath'] = os.path.join(partsDir, r['filename'])
			sL['resistors'] = resistorBin
			fzb = Template(file=getTemplatefile("bin_fzb.tmpl"), searchList = [sL])
			writeOutFileInSubDirectoryWithData(output, "bin", "%sBin.fzb" % (namedSet), fzb)
	
	except Usage, err:
		print >> sys.stderr, sys.argv[0].split("/")[-1] + ": " + str(err.msg)
		print >> sys.stderr, "\t for help use --help"
		return 2


if __name__ == "__main__":
	sys.exit(main())
	


# usage:
#    svnremove.py -d [directory]
#    
#    directory is a folder containing .svn directories.
#    recursively delete all .svn folders

import getopt, sys, os, re
import shutil, stat

def usage():
    print """
usage:
    svnremove.py -d [directory]
    
    directory is a folder containing .svn directories.  
    recursively delete all .svn folders
    """
    
    
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputDir = None
    findtext = ""
    gotnot = 0
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if(not(outputDir)):
        usage()
        sys.exit(2)
     
    
    for root, dirs, files in os.walk(outputDir, topdown=False):
        for dirname in dirs:
            if dirname == ".svn":  
                removeall(os.path.join(root, dirname))
                print "removing", os.path.join(root, dirname)

               
def removeall(top):
    for root, dirs, files in os.walk(top, topdown=False):
        for name in files:
            fullname = os.path.join(root, name)
            fileAtt = os.stat(fullname)[0]
            if (not fileAtt & stat.S_IWRITE):
                # File is read-only, so make it writeable
                os.chmod(fullname, stat.S_IWRITE)
            os.remove(fullname)
        for name in dirs:
            os.rmdir(os.path.join(root, name))	
    os.rmdir(top)
    
if __name__ == "__main__":
    main()




#	TODO:
#		check for incomplete parts (missing views)
#       check if svgs are plain copies of already existing ones and link them instead?
#       check for conflicting names

# lots of borrowing from http://code.activestate.com/recipes/252508-file-unzip/

import getopt, sys, os, os.path, re, zipfile
    
def usage():
    print """
usage:
    fzpzclean.py -f [fzpz directory] -d [output directory] -o [core | contrib | user] (-r)

    -f   directory of input fzpzs
    -d   directory for output of the cleanup process
    -o   parent folder name, depending on fzp category
    -r   optional: take the fzpz filename to rename the fzp filename

Unzips fzpz files into the output directory and cleans up file names and references.
"""
    
        
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:d:o:r", ["help", "fzpzs", "directory", "output", "rename"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    inputdir = None
    outputdir = None
    outputPrefix = None
    rename = False
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--fzpzs"):
            inputdir = a
        elif o in ("-d", "--directory"):
            outputdir = a
        elif o in ("-o", "--output"):
            outputPrefix = a
        elif o in ("-r", "--rename"):
            rename = True
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if(not(inputdir)):
        usage()
        sys.exit(2)

    if(not(outputPrefix)):
        usage()
        sys.exit(2)

    if(not(outputdir)):
        usage()
        sys.exit(2)

    if not outputdir.endswith(':') and not os.path.exists(outputdir):
        os.mkdir(outputdir)
   
    for fn in os.listdir(inputdir):
        if fn.endswith('.fzpz'):
                print fn
                file = os.path.join(inputdir, fn)
                zf = zipfile.ZipFile(file)

                # create directory structure to house files
                createstructure(file, outputdir, outputPrefix)

                # record svg renamings and fzp location for fixing paths later
                svgrenames = []
                fzplocation = None

                # extract files to directory structure
                for i, name in enumerate(zf.namelist()):
                        if not name.endswith('/'):

                                # identify file type
                                ending = name.split('.')[-1]
                                if ending == 'svg':
                                        pass
                                elif ending == 'fzp':
                                        pass
                                else:
                                        print "WARNING wrong file type:", name
                                        return

                                # sort files into subdirectories
                                subdir = None
                                if ending == 'fzp':
                                        subdir = outputPrefix
                                elif name.find('icon') >= 0:
                                        subdir = 'svg/' + outputPrefix + '/icon'
                                elif name.find('pcb') >= 0:
                                        subdir = 'svg/' + outputPrefix + '/pcb'
                                elif name.find('schem') >= 0:
                                        subdir = 'svg/' + outputPrefix + '/schematic'
                                elif name.find('bread') >= 0:
                                        subdir = 'svg/' + outputPrefix + '/breadboard'

                                # remove junk from filenames
                                outname = name
                                outname = re.sub('^svg\.((icon)|(breadboard)|(schematic)|(pcb))\.', '', outname, 1)
                                refname = outname  # this is how the svgs are referenced in the fzp
                                outname = re.sub('^part\.', '', outname, 1)
                                outname = re.sub('((__)|(_))[0-9a-fA-F]{32}', '', outname)
                                outname = re.sub('((__)|(_))[0-9a-fA-F]{27}', '', outname)
                                outname = re.sub('((__)|(_))[0-9a-fA-F]{20}', '', outname)
                                outname = re.sub('((__)|(_))((icon)|(breadboard)|(schematic)|(pcb))', '', outname)
                                # remove duplicate file endings
                                outname = re.sub('.' + ending, '', outname)
                                outname += '.' + ending

                                # optionally rename the fzp to that of the fzpz
                                if ending == 'fzp' and rename:
                                        base = os.path.basename(zf.filename)
                                        outname = os.path.splitext(base)[0]
                                        outname += '.fzp'

                                # write new files
                                filelocation = os.path.join(outputdir, subdir, outname)
                                outfile = open(filelocation, 'wb')
                                outfile.write(zf.read(name))
                                outfile.flush()
                                outfile.close()

                                # store paths for reference update
                                if ending == 'svg':
                                        svgrenames.append((refname, outname))
                                elif ending == 'fzp':
                                        fzplocation = filelocation

                # update svg references in fzp
                fzpfile = open(fzplocation, 'r+')
                s = fzpfile.read()
                for svgre in svgrenames:
                        if s.find(svgre[0]) == -1:
                                print "WARNING reference could not be found:", svgre[0]
                        s = s.replace(svgre[0], svgre[1])
                fzpfile.seek(0)
                fzpfile.truncate()
                fzpfile.write(s)
                fzpfile.flush()
                fzpfile.close()


        
def createstructure(file, dir, outputPrefix):
    # makedirs(listdirs(file), dir)
    dirs = [outputPrefix, 'svg/' + outputPrefix + '/icon', 'svg/' + outputPrefix + '/breadboard', 'svg/' + outputPrefix + '/schematic', 'svg/' + outputPrefix + '/pcb']
    makedirs(dirs, dir)


def makedirs(directories, basedir):
    """ Create any directories that don't currently exist """
    for dir in directories:
        curdir = os.path.join(basedir, dir)
        if not os.path.exists(curdir):
            os.makedirs(curdir)

def listdirs(file):
    """ Grabs all the directories in the zip structure
    This is necessary to create the structure before trying
    to extract the file to it. """
    zf = zipfile.ZipFile(file)

    dirs = []

    for name in zf.namelist():
        if name.endswith('/'):
            dirs.append(name)

    dirs.sort()
    return dirs


if __name__ == "__main__":
    main()






# usage:
#	fzzrename.py -f [fzz original path] -t [fzz new path] 
#	renames fzz file as well as internal fz file.  if the -t parameter is not a directory, the fzz is renamed but not copied

# lots of borrowing from http://code.activestate.com/recipes/252508-file-unzip/

import getopt, sys, os, os.path, re, zipfile, shutil
    
def usage():
    print """
usage:
    fzzrename.py -f [fzz original path] -t [fzz new path] 
    renames fzz file as well as internal fz file.  if the -t parameter is not a directory, the fzz is renamed but not copied
    """
    
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:t:", ["help", "from", "to"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    fromName = None
    toName = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--from"):
            fromName = a
        elif o in ("-t", "--to"):
            toName = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if(not(fromName)):
        usage()
        sys.exit(2)
        
    if not(fromName.endswith(".fzz")):
        print "-f is not .fzz"
        usage()
        sys.exit(2)
        
    if(not(toName)):
        usage()
        sys.exit(2)

    if not(toName.endswith(".fzz")):
        print "-t is not .fzz"
        usage()
        sys.exit(2)

    outputDir = os.path.dirname(toName)
    if len(outputDir) == 0:
        outputDir = os.path.dirname(fromName)
        toName = outputDir + "/" + toName
    else:
        if not outputDir.endswith(':') and not os.path.exists(outputDir):
            os.mkdir(outputDir)
   
    tempDir = outputDir + "/" + "___temp___"
    shutil.rmtree(tempDir, 1)
    os.mkdir(tempDir)
    
    zf = zipfile.ZipFile(fromName)
    zf.extractall(tempDir)
    
    fromBaseName = os.path.basename(fromName)
    toBaseName = os.path.basename(toName)
    
    fromFzName = os.path.splitext(fromBaseName)[0] + ".fz"
    toFzName = os.path.splitext(toBaseName)[0] + ".fz"
    
    print os.path.join(tempDir, fromFzName), os.path.join(tempDir, toFzName)
    os.rename(os.path.join(tempDir, fromFzName), os.path.join(tempDir, toFzName))
    
    # helpful examples in http://www.doughellmann.com/PyMOTW/zipfile/
    try:
        import zlib
        compression = zipfile.ZIP_DEFLATED
    except:
        compression = zipfile.ZIP_STORED

    zf = zipfile.ZipFile(outputDir + "/" + toBaseName, mode='w')
    for fn in os.listdir(tempDir):
        zf.write(os.path.join(tempDir, fn), fn, compression)
    zf.close()

    shutil.rmtree(tempDir, 1)


if __name__ == "__main__":
    main()







import getopt, sys, os, re
    
def usage():
    print """
usage:
    schemmatch.py -f <fritzing parts folder>
    
    looks for all svgs in fritzing parts/whatever and tries to find a matching obsolete/0.3.schem. file  
    ignore the ones that start with sparkfun-
    """
        
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "f:", ["fritzing"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    
    fritzingDir = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--fritzing"):
            fritzingDir = a
        else:
           print "unhandled option", o
           usage()
           return
            
    if not fritzingDir:
        usage()
        return
        
    for root, dirs, files in os.walk(fritzingDir, topdown=False):
        for name in files:
            if not name.endswith("svg"):
                continue
            if name.startswith("sparkfun-"):
                continue
                
            fullname = os.path.join(root, name)
            pathlist = fullname.split(os.sep)
            if pathlist[-4] != "svg":
                continue
                
            if pathlist[-3] == "obsolete":
                continue
            
            if pathlist[-2] != "schematic":
                continue
            
            pathlist[-3] = "obsolete"
            pathlist[-1] = "0.3.schem." + pathlist[-1]
            
            try:
                with open(os.sep.join(pathlist)):
                    pass
            except IOError:
                print "no 0.3.schem for", fullname
            

if __name__ == "__main__":
    main()




# usage:
#    copperNoSilkscreen.py -d <directory> 
#    
#    <directory> is a folder, with subfolders, containing .fzp files.  In each fzp file in the directory or its children
#    look for "copper" and "silkscreen"

import getopt, sys, os, re
    
def usage():
    print """
usage:
    copperNoSilkscreen.py -d [directory]
    
    directory is a folder containing .fzp files.  
    In each fzp file in the directory or its subfolders,
    look for "copper" and "silkscreen".
    """
    
    
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    outputDir = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if(not(outputDir)):
        usage()
        sys.exit(2)
        
    
    for root, dirs, files in os.walk(outputDir, topdown=False):
        for filename in files:
            if (filename.endswith(".fzp")):  
                infile = open(os.path.join(root, filename), "r")
                fzp = infile.read();
                infile.close();
                copperMatch = re.search('copper', fzp)
                silkscreenMatch = re.search('silkscreen', fzp)
                
                if (copperMatch == None):
                    print "{0} {1}".format(os.path.join(root, filename), "no copper")
                elif (silkscreenMatch == None):
                    print "{0} {1}".format(os.path.join(root, filename), "no silkscreen")

                
  
    
if __name__ == "__main__":
    main()




#!/usr/bin/python
# -*- coding: utf-8 -*-

# lots of borrowing from http://code.activestate.com/recipes/252508-file-unzip/

import getopt, sys, os, os.path, re, zipfile, shutil, xml.dom.minidom, xml.dom
import xml.etree.ElementTree as ET
    
def usage():
    print """
usage:
    fix.py -f [from directory]


For each fzz file in the from directory, remove all .ino files inside the fzz and <program> elements in the internal fz file.
if the fzz has a matching .ino file (external, in the same folder) then copy the ino file inside the fzz and add a <program> element to the fz
Afterwards you must delete the  external.ino files 
Probably safest to make a copy of the from directory first
"""
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "f:", ["from"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    
    fromDir = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-f", "--from"):
            fromDir = a
        else:
            print "unhandled option",o
            usage()
            return
    
    if not(fromDir):
        print "missing 'from' argument"
        usage()
        return
        
    try:
        import zlib
        compression = zipfile.ZIP_DEFLATED
    except:
        compression = zipfile.ZIP_STORED

    for root, dirs, files in os.walk(fromDir, topdown=False):
        for fzz in files:
            if not fzz.endswith('.fzz'):
                continue
                
            #print fzz
            fzzpath = os.path.join(root, fzz)
            inopath = fzzpath.replace(".fzz", ".ino")
            ino = os.path.basename(inopath)
            gotIno = os.path.isfile(inopath)
            
            tempDir =fromDir + os.sep + "___temp___"
            shutil.rmtree(tempDir, 1)
            os.mkdir(tempDir)

            zf = zipfile.ZipFile(fzzpath)
            zf.extractall(tempDir)
            zf.close()
                        
            modified = False
            
            print fzzpath
        
            # remove old ino files first
            for fino in os.listdir(tempDir):
                if fino.endswith(".ino"):
                    print "    remove old", fino
                    os.remove(os.path.join(tempDir, fino))
                    modified = True

            for fz in os.listdir(tempDir):                    
                if not fz.endswith(".fz"):
                    continue
                    
                try: 
                    fzpath = os.path.join(tempDir, fz)
                    tree = ET.parse(fzpath)
                    top = tree.getroot()
                    programs = []
                    for ps in top.findall('programs'):
                        programs.append(ps)
                    for ps in programs:
                        fzmodified = True
                        print "    remove old programs"
                        top.remove(ps)
                    
                    if gotIno:
                        shutil.copyfile(inopath, os.path.join(tempDir, ino))
                        ps = ET.SubElement(top, "programs")
                        ps.set("pid", "")
                        program = ET.SubElement(ps, "program")
                        program.set("language", "Arduino")
                        program.set("programmer", "")
                        program.text = ino
                        fzmodified = True
                        print "    copy", ino
                    
                    if fzmodified:
                        modified = True
                        tree.write(fzpath)
                except:
                    print "exception", fzpath, sys.exc_info()[0]
                    return                   

            if modified:
                os.remove(fzzpath)
                zf = zipfile.ZipFile(fzzpath, mode='w')
                for fn in os.listdir(tempDir):
                    zf.write(os.path.join(tempDir, fn), fn, compression)
                zf.close()

            shutil.rmtree(tempDir, 1)

if __name__ == "__main__":
    main()






import os, os.path

inputDir = "C:\\Users\\joev\\Documents\\AAA\\Fritzing\\scripting begin\\input"
outputDir = "C:\\Users\\joev\\Documents\\AAA\\Fritzing\\scripting begin\\output\\"

def remove_namespace(doc, namespace):


    ns = u'{%s}' % namespace
    nsl = len(ns)
    for tree in doc.getiterator():
        if tree.tag.startswith(ns):
            tree.tag = tree.tag[nsl:]


def myFilter(svgFilename, filename):

	#print "im myfilter",svgFilename
	import xml.etree.ElementTree as ET  #import ElementTree
	parser = ET.XMLParser(encoding="utf-8")
	tree = ET.parse(svgFilename, parser)
	remove_namespace(tree, u'http://www.w3.org/2000/svg') #call remove_namespace for cleaning the element-tags
	svgRoot = tree.getroot()
	# print "tree", tree

	parent_map = dict((c, p) for p in tree.getiterator() for c in p) #make a dictonary with a map of the hole file 
	#it has the childs and the its parents stored

	coppers = set() #make a set (set is like a list but without doubled same entries)
	for child in parent_map.values():
		removed = False
		# print "child", child
		id = child.attrib.get("id")
		if id and id.startswith('copper'): #get the child ids to decide if we need to clean or not
			coppers.add(child)
			
			
		paths = list() 
		
		for copper in coppers:
			paths.extend(copper.findall("path")) #writes the list where are path-elemnts are in that are in a parent called "copper"
			
			#print "paths", paths
			for path in paths:
				#print "path", path
				#print "parent_map", parent_map
				id = path.attrib.get("id")
				
				if id is None:
					try:
						#print "id", id
						parent_map[path].remove(path) #remove the child from his parent
						removed = True

					except:
						 continue#print "fuck", path, path.tag, parent_map.get(path)
				
				else:
					#print "id", id
					removed = False

			if removed: #write the new file in a new output directory
				svgRoot.set("xmlns:svg", "http://www.w3.org/2000/svg")
				svgRoot.set("xmlns", "http://www.w3.org/2000/svg")
				#print "root", root, "tree",tree, "filename", svgFilename
				outFilename = outputDir + filename
				#print "outputdir", outputDir, "outFilename", outFilename
				
				tree.write(outFilename)
				with open (outFilename, "r+") as f:
					old = f.read()
					f.seek(0)
					f.write('<?xml version="1.0" encoding="UTF-8" standalone="no"?>\n' + '<!-- Created with Fritzing (http://www.fritzing.org/) -->' + '\n' + old)
					f.close()        
				print "finised:", filename
				

def main(): 
	for root, dirs, files in os.walk(inputDir, topdown=True): #go to the dicrectory and get all files in a list called "files"
		for filename in files:
			if filename.endswith(".svg"):  #if it is an svg -> go on
				#print "im main"
				svgFilename = os.path.join(root, filename)
				#print root, filename
				myFilter(svgFilename, filename) #go to function myFilter
				continue
				



if __name__ == "__main__":
        main()





import getopt, sys, os, os.path, re, xml.dom.minidom, xml.dom
    
def usage():
        print """
usage:
    cleanschem.py -d [svg folder]
    cleans files with multiple copies of internal rects and labels
"""
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
        
    dir = None
            
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            dir = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            print "unhandled option", o
            usage()
            return
        
            
    if dir == None:
        print "missing directory argument"
        usage()
        return
          
            
    for root, dirs, files in os.walk(dir, topdown=False):
        for filename in files:
            if not filename.endswith(".svg"): 
                continue
                
            svgFilename = os.path.join(root, filename)
            handlesvg(svgFilename)

                
def handlesvg(svgFilename):
    try:
        dom = xml.dom.minidom.parse(svgFilename)
    except xml.parsers.expat.ExpatError, err:
        print str(err), svgFilename
        return
        
    #print svgFilename
    rcount = 0
    toDelete = []
    svg = dom.documentElement
    rectNodes = svg.getElementsByTagName("rect")
    for rect in rectNodes:
        if rect.getAttribute("class") == "interior rect":
            rcount += 1
            if rcount > 1:
                toDelete.append(rect)
                print "\t", rect.toxml("UTF-8")
                
    #print rcount, toDelete
    
    textNodes = svg.getElementsByTagName("text")
    labels = []
    fix = False
    for text in textNodes:
        id = text.getAttribute("id")
        if id.startswith("label"):
            labels.append(text)
            if "_" in id:
                #fix = True
                #text.setAttribute("id", id.replace("_", ""))
                #print "id", id
                pass
            
    if len(labels) > 0:
        for ix in range(len(labels)):
            labeli = labels[ix]
            for jx in range(ix + 1, len(labels)):
                labelj = labels[jx]
                if labeli.toxml("UTF-8") == labelj.toxml("UTF-8"):
                    toDelete.append(labeli)
                    print "\t", labeli.toxml("UTF-8")
                    break
                
    for node in toDelete:
        node.parentNode.removeChild(node)
        
    if len(toDelete) > 0 or fix:
        print "fixing", svgFilename
        outfile = open(svgFilename, 'wb')
        s = dom.toxml("UTF-8")
        outfile.write(s)
        outfile.close()                    
    


            
if __name__ == "__main__":
        main()






#defines attribute constants for gEDA PCB parameters
#note - some tags accept variable numbers of arguments

tagMap = {}
tagMap["Element"] = [["SFlags","Desc","Name","Value","MX","MY","TX","TY",\
            "TDir","TScale","TSFlags"],\
           ["NFlags","Desc","Name","Value","TX","TY","TDir","TScale","TNFlags"],\
           ["NFlags","Desc","Name","TX","TY","TDir","TScale","TNFlags"],\
           ["Desc","Name","TX","TY","TDir","TScale","TNFlags"]]
tagMap["Pin"] = [["rX","rY","Thickness","Clearance","Mask","Drill","Name",\
        "Number","SFlags"],\
       ["aX","aY","Thickness","Drill","Name","Number","NFlags"],\
       ["aX","aY","Thickness","Drill","Name","NFlags"],\
       ["aX","aY","Thickness","Name","NFlags"]]
tagMap["Pad"] = [["rX1","rY1","rX2","rY2","Thickness","Clearance","Mask",\
        "Name","Number","SFlags"],\
       ["rX1","rY1","rX2","rY2","Thickness","Name","Number","NFlags"],\
       ["rX1","rY1","rX2","rY2","Thickness","Name","NFlags"]]
tagMap["ElementLine"] = [["X1","Y1","X2","Y2","Thickness"]]
tagMap["ElementArc"] = [["X","Y","Width","Height","StartAngle","DeltaAngle",\
        "Thickness"]]
tagMap["Attribute"] = [["Name","Value"]] 
tagMap["Mark"] = [["Y","X"]]

def getNextAttr(attributes):
    """
    searches through a string to find the next space separated attribute,
    while being careful not to mess up double-quoted strings.
    returns a 2-tuple with the attribute followed by the remaining string
    """
    attributes = attributes.strip()
    if(attributes[0] == '"'):
        attributes = attributes[1:]
        attr, tmp, trimmed = attributes.partition('"')
    else:
        tmp = attributes.split(None, 1)
        if(len(tmp)<2):
            return tmp[0], ""
        attr = tmp[0]
        trimmed = tmp[1]
    return attr, trimmed

def makeDict(tag, attributes):
    """
    splits a list of attributes given a tag and returns a dict with 
    attributed names as keys.
    If the tag is not in the lookup table, an empty dict is returned.
    """
    if(not(tagMap.has_key(tag))):
        return {}
        
    #attrList = attributes.split()
    
    attrList = []
    while(attributes != ""):
        attr, attributes = getNextAttr(attributes)
        attrList.append(attr)
    
    tagList = None
    for selection in tagMap[tag]:
        if len(selection) == len(attrList):
            tagList = selection
    
    #no possible mapping found - should throw an exeption actually
    if(not(tagList)):
        return {}
    
    #attrListCleaned = []
    
    #strip quotes
    #for attr in attrList:
    #    if attr[0] == '"':
    #        tmp1, tmp2, attr = attr.partition('"')
    #        attr, tmp1, tmp2 = attr.rpartition('"')
    #    attrListCleaned.append(attr)
    
    #merge the two into a dictionary
    rtn = {}
    for i in xrange(len(tagList)):
        rtn[tagList[i]] = attrList[i]
    
    return rtn



# usage:
#	    copper1fzp.py -d [fzp folder]
#       adds a copper1 layer if there isn't one found already.

import getopt, sys, os, os.path, re, xml.dom.minidom, xml.dom
    
def usage():
        print """
usage:
    connectors_misnumbered.py -d [fzp folder]
    checks that connectors with integer names are correctly mapped to connector numbers
"""
    
        
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
        
    dir = None
            
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            dir = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            assert False, "unhandled option"
            
    if(not(dir)):
        usage()
        return       

    pattern = r'(\d+)'
    numberFinder = re.compile(pattern, re.IGNORECASE)
            
    for root, dirs, files in os.walk(dir, topdown=False):
        for filename in files:
            if not filename.endswith(".fzp"): 
                continue
                
            fzpFilename = os.path.join(root, filename)
            try:
                dom = xml.dom.minidom.parse(fzpFilename)
            except xml.parsers.expat.ExpatError, err:
                print str(err), fzpFilename
                continue
                
            fzp = dom.documentElement
            connectors = fzp.getElementsByTagName("connector")
            gotInt = False
            for connector in connectors:
                try:
                    intname = int(connector.getAttribute("name"))
                    gotInt = True
                except:
                    continue
                    
            if not gotInt:
                continue
                
            idZero = False
            for connector in connectors:
                try:
                    id = connector.getAttribute("id")
                    match = numberFinder.search(id)
                    if match == None:
                        continue
        
                    if match.group(1) == '0':
                        idZero = True
                        break
                except:
                    continue
                
            nameZero = False
            for connector in connectors:
                if connector.getAttribute("name") == "0":
                    nameZero = True
                    break
                    
               
            mismatches = []
            for connector in connectors:
                idInt = 0
                nameInt = 0
                try:
                    id = connector.getAttribute("id")
                    match = numberFinder.search(id)
                    if match == None:
                        continue
        
                    idInt = int(match.group(1))
                    nameInt = int(connector.getAttribute("name"))
                    
                except:
                    continue
                    
                mismatch = False
                if nameZero and idZero:
                    mismatch = (idInt != nameInt)
                elif nameZero:
                    mismatch = (idInt != nameInt + 1)
                elif idZero:
                    mismatch = (idInt + 1 != nameInt)
                else:
                    mismatch = (idInt != nameInt)
                if mismatch:
                    mismatches.append(connector)
            
            if len(mismatches) > 0:
                print fzpFilename, nameZero, idZero
                for connector in mismatches:
                    strings = connector.toxml().split("\n")
                    print strings[0]
                print
                    
            
if __name__ == "__main__":
        main()






# usage:
#	fpgrabber.py -u [url to crawl] -d [output directory]
#	 recursively crawl the url and download fp files into the output directory.

# code borrowed from http://theanti9.wordpress.com/2009/02/14/python-web-crawler-in-less-than-50-lines/

import getopt, sys, os, os.path, re, urllib2, urlparse
    
def usage():
    print """
usage:
    fpgrabber.py -u [url to crawl] -d [output directory]
    recursively crawl the url and download fp files into the output directory.
    """
 
linkregex = re.compile('<a\s*href=[\"\'](.[^\"\']+)[\"\']')
redirectex = re.compile('content=[\"\']\d+;\s*URL=(.[^\"\']+)[\"\']')
tocrawl = []
crawled = set([])
outputdir = None
inputurl = None
 
def main():
    global inputurl, outputdir
    
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hu:d:", ["help", "url", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-u", "--url"):
            inputurl = a
        elif o in ("-d", "--directory"):
            outputdir = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
    
    if(not(inputurl)):
        usage()
        sys.exit(2)

    if(not(outputdir)):
        usage()
        sys.exit(2)

    if not outputdir.endswith(':') and not os.path.exists(outputdir):
        os.mkdir(outputdir)
        
    if not inputurl.endswith("/"):
        #footprint site seems to require this
        inputurl = inputurl + "/"
       
    tocrawl.append(inputurl)
    crawl()
    
def savefp(url):
    print "   saving " + url
    filehandle = None
    try:
        filehandle = urllib2.urlopen(url)
    except Exception as inst:
        print inst
        pass
        
    if (not filehandle):
        print "unable to open 1 " + url
        return
    
    p = url.split('/')
    fn = p[len(p) - 1]
    fn = fn[:-3]
    fn = urllib2.unquote(fn)
    f = open(os.path.join(outputdir, fn),'w')
    if (not f):
        print "unable to save " + fn
        filehandle.close()
        return
        
    for lines in filehandle.readlines():
        f.write(lines)

    f.close()
    filehandle.close()       
    return

def crawl():
        
    while len(tocrawl) > 0:
        crawling = tocrawl.pop()
        print "crawling " + crawling
          
        try:
            response = urllib2.urlopen(crawling)
        except:
            print "failed to open " + crawling
            return
            
        #print "code " + str(response.getcode())
        #url = urlparse.urlparse(response.geturl())
        if crawling != response.geturl():
            #this never gets triggered by the footprint site, geturl doesn't seem to return redirects
            print "changed " + crawling + " " + response.geturl()
            
        #info = response.info()
        #print info
        
        msg = response.read()
        response.close()
        links = linkregex.findall(msg)
        if (len(links) == 0):
            if (msg.find("<meta") >= 0):
                links = redirectex.findall(msg)
                #print "     redirect "  + msg + " " + str(len(links))
            
        crawled.add(crawling)
        for link in (links.pop(0) for _ in xrange(len(links))):    
            newlink = urlparse.urljoin(crawling, urllib2.quote(link, ":/?%+"))
            #print "        link " + newlink + " " + link
            if (newlink.endswith("fp?dl")):
                savefp(newlink)
                continue
                
            if (newlink.find(inputurl) >= 0) and (newlink not in crawled):   
                tocrawl.append(newlink)
   
if __name__ == "__main__":
    main()






#this object takes an fzfp (an xml-ized gEDA footprint) and makes
# it into a Fritzing footprint SVG.
import xml.dom.minidom

#TODO: rounded pads

class SVGGen(xml.dom.minidom.Element):
    def __init__(self, xmlFile, debug=False):
        self.debug = debug
        self._svg = xml.dom.minidom.Document
    
    def getSvgFile(self):
    # spits out a string with the generated SVG
        return self._svgFile
    
    def _drawNode(self, node):
    # renders a node to SVG
        self._debug("drawing node: ")
        tag = node.nodeName.lower()
        
        if tag == "pin":
            self._debug("pin")
            self._drawPin(node)
        elif tag == "pad:
            self._debug("pad")
            self._drawPad(node)
        elif tag == "elementline":
            self._debug("element line")
            self._drawElementLine(node)
        elif tag == "elementarc":
            self._debug("element arc")
            self._drawElementArc(node)
        elif tag == "mark":
            self._debug("mark")
            self._drawMark(node)
        else:
            #TODO: throw an exception
            self._debug("error - cannot render unrecognized tag")
        
    def _drawPin(self, node):
    # render a pin to SVG
        pass
        
    def _drawPad(self, node):
    # render a copper pad to SVG
        pass
        
    def _drawElementLine(self, node):
    # render a silkscreen line
        pass
        
    def _drawElementArc(self, node):
    # render a silkscreen circular arc
        pass
        
    def _drawMark(self, node):
    # render the center point mark
        pass
        
    def minMax(self, x, y, strokeWidth):
    """
    check to see if the coordinates are within the viewbox
    adjusts if necessary
    """
        pass
        
    def _debug(self, msg):
        if self.debug:
            print msg




import getopt, sys, os, os.path, re, xml.dom.minidom, xml.dom
    
def usage():
        print """
usage:
    unzeroradius.py -d [svg folder]
    if a file has <circle r='0' replace with r='.00000000001'
"""
    
        
           
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit(2)
        
    dir = None
            
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            dir = a
        elif o in ("-h", "--help"):
            usage()
            sys.exit(2)
        else:
            assert False, "unhandled option"
            
    if(not(dir)):
        usage()
        sys.exit(2)            
            
    for root, dirs, files in os.walk(dir, topdown=False):
        for filename in files:
            if not filename.endswith(".svg"): 
                continue
                
            svgFilename = os.path.join(root, filename)
            try:
                dom = xml.dom.minidom.parse(svgFilename)
            except xml.parsers.expat.ExpatError, err:
                print str(err), svgFilename
                continue
                
            svg = dom.documentElement
            circleNodes = svg.getElementsByTagName("circle")

            count = 0
            for circle in circleNodes:
                r = circle.getAttribute("r")
                if r == "0":
                    circle.setAttribute("r", "0.00000000001")
                    count += 1
                    
            if count == 0:
                #print ".",
                continue
                
            print "got zero", svgFilename
            outfile = open(svgFilename, 'wb')
            s = dom.toxml("UTF-8")
            outfile.write(s)
            outfile.flush()
            outfile.close()                        
            
if __name__ == "__main__":
        main()






# usage:
#    copperNoSilkscreen.py -d <directory> 
#    
#    <directory> is a folder, with subfolders, containing .fzp files.  In each fzp file in the directory or its children
#    look for "copper" and "silkscreen"

import getopt, sys, os, re
    
def usage():
    print """
usage:
    pathNoText.py -d [directory]
    
    directory is a folder containing .svg files.  
    In each fzp file in the directory or its subfolders,
    look for "<path>" and no "<text>".
    """
    
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hd:", ["help", "directory"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    outputDir = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--directory"):
            outputDir = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            assert False, "unhandled option"
    
    if not(outputDir) :
        usage()
        return
        
    
    for root, dirs, files in os.walk(outputDir, topdown=False):
        for filename in files:
            if filename.endswith(".svg"):  
                infile = open(os.path.join(root, filename), "r")
                svg = infile.read()
                infile.close()
                textMatch = '<text' in svg
                pathMatch = '<path' in svg
                
                if not(textMatch) and pathMatch:
                    print "{0} {1}".format(os.path.join(root, filename), "path no text")

                
    
if __name__ == "__main__":
    main()






import getopt, sys, os, re, time, shutil
    
def usage():
    print """
usage:
    copyif.py -s <source directory> -d <dest directory>
    
    copies files from source into dest, if the file already exists in dest
    
    """
    
    
       
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hs:d:", ["help", "source","dest"])
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        return
    
    source = None
    dest = None
    
    for o, a in opts:
        #print o
        #print a
        if o in ("-d", "--dest"):
            dest = a
        elif o in ("-s", "--source"):
            source = a
        elif o in ("-h", "--help"):
            usage()
            return
        else:
            print "unknown option", o
            return
    
    if not dest:
        print "missing -d (dest) argument"
        usage()
        return
     
    if not source:
        print "missing -s (source) argument"
        usage()
        return
     
    fs = []
    for f in os.listdir(source):
        df = os.path.join(dest, f)
        if os.path.exists(df) and os.path.isfile(df):
            shutil.copy(os.path.join(source,f), df)
    
if __name__ == "__main__":
    main()





import sys, os, os.path, re, xml.dom.minidom, xml.dom,  optparse
    
def usage():
        print """
usage:
    listfamilies.py -d [fzp folder] {-p [prefix] }
    lists families and optionally provides a prefix
"""
    
        
           
def main():
    parser = optparse.OptionParser()
    parser.add_option('-d', '--directory', dest="dir" )
    parser.add_option('-p', '--prefix', dest="prefix" )
    (options, args) = parser.parse_args()
        
    if not options.dir:
        usage()
        parser.error("dir argument not given")
        return       
            
    names = []
    for root, dirs, files in os.walk(options.dir, topdown=False):
        for filename in files:
            if not filename.endswith(".fzp"): 
                continue
                
            fzpFilename = os.path.join(root, filename)
            try:
                dom = xml.dom.minidom.parse(fzpFilename)
            except xml.parsers.expat.ExpatError, err:
                print str(err), fzpFilename
                continue
                
            fzp = dom.documentElement
            properties = fzp.getElementsByTagName("property")
            for property in properties:
                if property.getAttribute("name") == 'family':
                    value = getText(property.childNodes)
                    if not value in names:
                        names.append(value)
                        
                    if options.prefix != None:
                        for node in property.childNodes:
                            if node.nodeType == node.TEXT_NODE:
                                if not (options.prefix.lower() in node.data.lower()):
                                    node.data = options.prefix + " " + node.data
                                    outfile = open(fzpFilename, 'wb')
                                    s = dom.toxml("UTF-8")
                                    outfile.write(s)
                                    outfile.close()                    

                                break
                    break
                    
    names.sort()
    for name in names:
        print name

                
def getText(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    return ''.join(rc)                 

                    
            
if __name__ == "__main__":
        main()



